/******************************************************************************
 * macro.h
 * SASS Compiler for CoreTone, sfx + instrument parsing and pack generation.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in ctcc.c
 ******************************************************************************/
#ifndef CTCC_SFX
#define CTCC_SFX

/******************************************************************************
 * Operating Parameters
 ******************************************************************************/
#define MACRO_PARSER_LINEBUF_LEN	2048

#define MACRO_PARSER_MACBUF_LEN		64
#define MACRO_PARSER_MACCHANNEL_LEN	4
#define MACRO_PARSER_MACSTREAM_LEN	1024

#define MACRO_PARSER_PRIORITY_MIN	-128
#define MACRO_PARSER_PRIORITY_MAX	127
#define MACRO_PARSER_LOOP_MIN		-128
#define MACRO_PARSER_LOOP_MAX		127
#define MACRO_PARSER_WAIT_MIN		0

/******************************************************************************
 * Macro Channels and Blocks (Channel Groups)
 ******************************************************************************/
typedef struct MacroChannel_s
{
	uint32_t uiSample;
	int32_t iAmClosed,iGotSample;

	int32_t iWasWaiting;
	uint32_t uiDelayAcc,uiDelayLen;

	uint8_t *pacStream;
	uint32_t uiScriptLen,uiBufferLen;
} MacroChannel_t;

typedef struct MacroBlock_s
{
	char *pszName;

	int32_t iInstrument,iPriority;
	int32_t iHaveNoteOff;
	uint32_t uiNoteOff;

	MacroChannel_t *pChannels;
	uint32_t uiChannels,uiChannelSlots;
} MacroBlock_t;

/* Macro Table
 *------------------------------------*/
extern MacroBlock_t *pMacros;
extern uint32_t uiMacros;

/******************************************************************************
 * Function Defines
 ******************************************************************************/
int32_t mac_parse(FILE *pSASS, char *pszFileName, int32_t iDoSFX, int32_t iNoSamples);
int32_t sfx_make(void);
int32_t ins_make(FILE *pDest, char *pszFileName);
void mac_close(void);

int32_t mac_addByte(MacroChannel_t *pChannel, uint8_t ucByte);
int32_t mac_prepEntry(MacroBlock_t *pMacro, int32_t iInstrument);
int32_t mac_get(char *pszName, uint32_t *puiNum, MacroBlock_t **ppMacro);

#endif
