/******************************************************************************
 * chanman.cpp
 * Channel status visualization.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdio.h>
#include <stdint.h>
#include <math.h>

#include "../types.h"
#include "../CoreTone/coretone.h"
#include "../CoreTone/channel.h"
#include "chanman.hpp"
#include "resource.h"

/* VS6's Commctrl doesn't provide a #def for "STATIC" window classes,
 * this was added as "WC_STATIC" in later versions.
 */
#ifndef WC_STATIC
	#define WC_STATIC					_T("STATIC")
#endif

/******************************************************************************
 * !!!!----                      LABEL TABEL                         ----!!!!
 ******************************************************************************/
const TCHAR szName_chInfoB[] = _T("Channel Info Border");
const TCHAR szName_chInfoL[] = _T("Channel Status");

const TCHAR szData_chLegend[] = _T("# - Status             "
								   "Type       "
								   "Priority         "
								   "Phase                  "
								   "Phase Adjust       "
								   "Vol         "
								   "Panning <L, R>");

const TCHAR szData_chMainS[] = _T("%01X - %s        ");
const TCHAR szData_chMainD[] = _T("%02X - %s        ");

const TCHAR *szData_chModeS[] = {
	_T("Idle            "), _T("Single Shot ->  "), _T("Looping <->     "),
	_T("Unknown         ")
};

const TCHAR szData_ptModeS[] = _T("s  ");
const TCHAR szData_ptModeI[] = _T("i  ");
const TCHAR szData_ptPri[] = _T("%d    ");

const TCHAR szData_chPhase[] = _T("$%04X.%04X    ");
const TCHAR szData_chPhaseAdj[] = _T("$%04X.%04X    ");
const TCHAR szData_chVolM[] = _T("%d        ");
const TCHAR szData_chVolL[] = _T("<  %d            ");
const TCHAR szData_chVolR[] = _T("   %d  >");

const TCHAR szName_ctEmpty[] = _T("");


/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
ChanMan::ChanMan(HWND hParent, CoreChannel_t *pCoreChannels,
                 CorePatch_t *pCorePatches, UINT uiChannels)
{
	mhParent = hParent;
	mpCoreChannels = pCoreChannels;
	mpCorePatches = pCorePatches;
	muiChannels = uiChannels;
	muiHeightM = (CSTAT_HEIGHT_PER_CHANNEL * (uiChannels + 1));
	muiHeightB = muiHeightM + 8;

	/**
	 * ---- BORDER AND LABEL ----
	 */
	midBorder = CSTAT_BORDER_ID;
	mhBorder = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_chInfoB,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_ETCHEDFRAME | SS_LEFT),		/* dwStyle */
		CSTAT_BORDER_X,							/* x */
		CSTAT_BORDER_Y,							/* y */
		CSTAT_BORDER_WIDTH,						/* nWidth */
		muiHeightB,								/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midBorder,						/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder, szName_ctEmpty);

	midLabel = CSTAT_LABEL_ID;
	mhLabel = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_chInfoL,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_LEFT),						/* dwStyle */
		CSTAT_LABEL_X,							/* x */
		CSTAT_LABEL_Y,							/* y */
		CSTAT_LABEL_WIDTH,						/* nWidth */
		CSTAT_LABEL_HEIGHT,						/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midLabel,						/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
}

ChanMan::~ChanMan(void)
{
	
}




/******************************************************************************
 * !!!!----                   WINPROC/UI HANDLERS                    ----!!!!
 ******************************************************************************/
/* LRESULT ChanMan::PaintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
 *----------------------------------------------------------------------------*/
LRESULT ChanMan::PaintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
{
	HFONT hFontNew,hFontPrv;
	COLORREF crWindow,crPrev,crActive;
	RECT rText;

	CoreChannel_t *pChannel;
	CorePatch_t *pPatch;
	int8p8_t scale_L,scale_R;
	uint8_t cL,cR;
	UINT uiX,uiY,uiZ;

	/**
	 * We're interested in the OPERATING MODE, FREQUENCY, and AMPLITUDE
	 * of each channel within the CoreTone driver.
	 */
	crWindow = GetSysColor(COLOR_3DFACE);
	SetBkMode(hDC, OPAQUE);
	SetBkColor(hDC, crWindow);
	hFontNew = (HFONT)GetStockObject(ANSI_VAR_FONT);

	hFontPrv = (HFONT)SelectObject(hDC, hFontNew);
	if(NULL != hFontPrv)
	{
		uiY = CSTAT_INFO_Y + 2;
		rText.top = uiY;
		rText.bottom = uiY + CSTAT_HEIGHT_PER_CHANNEL;
		rText.left = (CSTAT_INFO_X + 0);
		rText.right = (CSTAT_INFO_X + 472);
		DrawText(hDC, szData_chLegend, -1, &rText,
			(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

		uiY += CSTAT_HEIGHT_PER_CHANNEL;
		for(uiX = 0; uiX < muiChannels; uiX++)
		{
			pChannel = mpCoreChannels + uiX;
			pPatch = mpCorePatches + uiX;
			rText.top = uiY;
			rText.bottom = uiY + CSTAT_HEIGHT_PER_CHANNEL;
			if(eCHANNEL_MODE_OFF != pChannel->eMode)
			{
				scale_L.sWhole = pChannel->cVolMain * pChannel->cVolLeft;
				cL = 4 * ((scale_L.cPair.cHi & 0x80)
					? (scale_L.cPair.cHi * -1)
					: scale_L.cPair.cHi);
				scale_R.sWhole = pChannel->cVolMain * pChannel->cVolRight;
				cR = 4 * ((scale_R.cPair.cHi & 0x80)
					? (scale_R.cPair.cHi * -1)
					: scale_R.cPair.cHi);
				crActive = PALETTERGB(cL,0x00,cR);
			}
			else
			{
				crActive = GetSysColor(COLOR_3DSHADOW);
			}

			crPrev = GetTextColor(hDC);
			SetTextColor(hDC, crActive);


			/**
			 * ---- LABEL & MODE ----
			 */
			rText.left = (CSTAT_INFO_X + 0);
			rText.right = (CSTAT_INFO_X + 88);
			uiZ = (pChannel->eMode < eCHANNEL_MODE_FOOTER)
				? pChannel->eMode
				: eCHANNEL_MODE_FOOTER;
			sprintf(mszRender,
				(muiChannels > 16) ? szData_chMainD : szData_chMainS,
				uiX, szData_chModeS[uiZ]);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			/**
			 * ---- PATCH TYPE ----
			 */
			rText.left = (CSTAT_INFO_X + 88);
			rText.right = (CSTAT_INFO_X + 104);
			DrawText(hDC,
				(pPatch->iInstrument ? szData_ptModeI : szData_ptModeS),
				-1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			/**
			 * ---- PRIORITY AND PENDING DELAY ----
			 */
			rText.left = (CSTAT_INFO_X + 132);
			rText.right = (CSTAT_INFO_X + 188);
			sprintf(mszRender, szData_ptPri,
				pPatch->iPriority);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			/**
			 * ---- PHASE and PHASE ADJUSTMENT ----
			 */
			rText.left = (CSTAT_INFO_X + 188);
			rText.right = (CSTAT_INFO_X + 272);
			sprintf(mszRender, szData_chPhase,
				pChannel->phaseAcc.usPair.usHi,
				pChannel->phaseAcc.usPair.usLo);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			rText.left = (CSTAT_INFO_X + 272);
			rText.right = (CSTAT_INFO_X + 356);
			sprintf(mszRender, szData_chPhaseAdj,
				pChannel->phaseAdj.usPair.usHi,
				pChannel->phaseAdj.usPair.usLo);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			/**
			 * ---- VOLUME ----
			 */
			rText.left = (CSTAT_INFO_X + 356);
			rText.right = (CSTAT_INFO_X + 395);
			sprintf(mszRender, szData_chVolM,
				pChannel->cVolMain);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			rText.left = (CSTAT_INFO_X + 396);
			rText.right = (CSTAT_INFO_X + 464);
			sprintf(mszRender, szData_chVolL,
				pChannel->cVolLeft);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

			rText.left = (CSTAT_INFO_X + 428);
			rText.right = (CSTAT_INFO_X + 472);
			sprintf(mszRender, szData_chVolR,
				pChannel->cVolRight);
			DrawText(hDC, mszRender, -1, &rText,
				(DT_RIGHT | DT_WORDBREAK | DT_EXTERNALLEADING));


			uiY += CSTAT_HEIGHT_PER_CHANNEL;
			SetTextColor(hDC, crPrev);
		}

		SelectObject(hDC, hFontPrv);
	}

	return 0;
}

/* LRESULT ChanMan::InvalProc(HWND hWnd)
 *----------------------------------------------------------------------------*/
LRESULT ChanMan::InvalProc(HWND hWnd)
{
	RECT rInfo;

	rInfo.left = CSTAT_INFO_X;
	rInfo.right = (CSTAT_INFO_X + CSTAT_INFO_WIDTH);
	rInfo.top = CSTAT_INFO_Y;
	rInfo.bottom = CSTAT_INFO_Y + (muiHeightM + 5);

	InvalidateRect(hWnd, &rInfo, FALSE);
	return 0;
}
